{#
The MIT License (MIT)

Copyright (c) 2012-2019 Juan Luis Cano Rodríguez and the poliastro development team

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.
#}
{% extends "!layout.html" %}

{% block menu %}
    {{ super() }}

    {% if sidebar_external_links %}
        <p class="caption">
            <span class="caption-text">
            {% if sidebar_external_links_caption %}
                {{ sidebar_external_links_caption }}
            {% else %}
                External links
            {% endif %}
            </span>
        </p>
        <ul>
            {% for text, link in sidebar_external_links %}
                {% if hasdoc(link) %}
                    <li class="toctree-l1"><a href="{{ pathto(link) }}">{{ text }}</a></li>
                {% else %}
                    <li class="toctree-l1"><a href="{{ link }}">{{ text }}</a></li>
                {% endif %}
            {% endfor %}
        </ul>
    {% endif %}

{% endblock %}
