!     Copy the content of your .usr file in this .usr.f file
!
!     This subroutine is not added by makenek (as of Nek5000 v19), but is mandatory
      subroutine usrdat()
      implicit none

      return
      end
