{# A template for generating box files.

A box file is a description of a simple box mesh. It describes the spatial
dimension of the mesh, the number of fields, the domain extents and boundary
conditions. The box file is rendered using the write_box method of the Output
class.
-#}
{% set indent = 42 -%}
{{ dim.ljust(indent) }} spatial dimension (dim<0 will create box.re2)
{{ nb_fields.ljust(indent) }} number of fields
#========================================================
#
{% for comment in comments.splitlines() -%}
#    {{ comment }}
{% endfor -%}
#
#========================================================
#
Box
{% for comment, grid in grid_info.items() -%}
{{ grid.ljust(indent) }}{{comment}}
{% endfor -%}
